@@ -4,14 +4,15 @@ use strict;
 use warnings;
 use 5.010;
 
-no if $] >= 5.018, warnings => "experimental::smartmatch";
+no if $] >= 5.018, warnings => 'experimental::smartmatch';
 
-our $VERSION = '1.09';
+our $VERSION = '1.10';
 
 use Carp qw(confess cluck);
 use Encode qw(encode decode);
 use Travel::Status::DE::EFA::Line;
 use Travel::Status::DE::EFA::Result;
+use Travel::Status::DE::EFA::Stop;
 use LWP::UserAgent;
 use XML::LibXML;
 
@@ -104,8 +105,15 @@ sub new {
 			useProxFootSearch      => '0',
 			useRealtime            => '1',
 		},
+		developer_mode => $opt{developer_mode},
 	};
 
+	if ( $opt{full_routes} ) {
+		$self->{post}->{depType}                = 'stopEvents';
+		$self->{post}->{includeCompleteStopSeq} = 1;
+		$self->{want_full_routes}               = 1;
+	}
+
 	bless( $self, $class );
 
 	$ua->env_proxy;
@@ -123,7 +131,9 @@ sub new {
 		string => $self->{xml},
 	);
 
-	#say $self->{tree}->toString(1);
+	if ( $self->{developer_mode} ) {
+		say $self->{tree}->toString(1);
+	}
 
 	$self->check_for_ambiguous();
 
@@ -153,6 +163,10 @@ sub errstr {
 sub sprintf_date {
 	my ($e) = @_;
 
+	if ( $e->getAttribute('day') == -1 ) {
+		return;
+	}
+
 	return sprintf( '%02d.%02d.%d',
 		$e->getAttribute('day'),
 		$e->getAttribute('month'),
@@ -163,6 +177,10 @@ sub sprintf_date {
 sub sprintf_time {
 	my ($e) = @_;
 
+	if ( $e->getAttribute('minute') == -1 ) {
+		return;
+	}
+
 	return sprintf( '%02d:%02d',
 		$e->getAttribute('hour'),
 		$e->getAttribute('minute'),
@@ -230,6 +248,20 @@ sub check_for_ambiguous {
 	return;
 }
 
+sub identified_data {
+	my ($self) = @_;
+
+	my $xp_place
+	  = XML::LibXML::XPathExpression->new('//itdOdv/itdOdvPlace/odvPlaceElem');
+	my $xp_name
+	  = XML::LibXML::XPathExpression->new('//itdOdv/itdOdvName/odvNameElem');
+
+	my $e_place = ( $self->{tree}->findnodes($xp_place) )[0];
+	my $e_name  = ( $self->{tree}->findnodes($xp_name) )[0];
+
+	return ( $e_place->textContent, $e_name->textContent );
+}
+
 sub lines {
 	my ($self) = @_;
 	my @lines;
@@ -284,6 +316,40 @@ sub lines {
 	return @lines;
 }
 
+sub parse_route {
+	my ( $self, @nodes ) = @_;
+	my $xp_routepoint_date
+	  = XML::LibXML::XPathExpression->new('./itdDateTime/itdDate');
+	my $xp_routepoint_time
+	  = XML::LibXML::XPathExpression->new('./itdDateTime/itdTime');
+
+	my @ret;
+
+	for my $e (@nodes) {
+		my @dates = $e->findnodes($xp_routepoint_date);
+		my @times = $e->findnodes($xp_routepoint_time);
+
+		# note that the first stop has an arrival node with an invalid
+		# timestamp and the terminal stop has a departure node with an
+		# invalid timestamp.  sprintf_{date,time} return undef in these
+		# cases.
+		push(
+			@ret,
+			Travel::Status::DE::EFA::Stop->new(
+				arr_date => sprintf_date( $dates[0] ),
+				arr_time => sprintf_time( $times[0] ),
+				dep_date => sprintf_date( $dates[-1] ),
+				dep_time => sprintf_time( $times[-1] ),
+				name     => decode( 'UTF-8', $e->getAttribute('name') ),
+				name_suf => decode( 'UTF-8', $e->getAttribute('nameWO') ),
+				platform => $e->getAttribute('platformName'),
+			)
+		);
+	}
+
+	return @ret;
+}
+
 sub results {
 	my ($self) = @_;
 	my @results;
@@ -301,6 +367,10 @@ sub results {
 	my $xp_line  = XML::LibXML::XPathExpression->new('./itdServingLine');
 	my $xp_info
 	  = XML::LibXML::XPathExpression->new('./itdServingLine/itdNoTrain');
+	my $xp_prev_route
+	  = XML::LibXML::XPathExpression->new('./itdPrevStopSeq/itdPoint');
+	my $xp_next_route
+	  = XML::LibXML::XPathExpression->new('./itdOnwardStopSeq/itdPoint');
 
 	if ( $self->{results} ) {
 		return @{ $self->{results} };
@@ -336,11 +406,21 @@ sub results {
 		my $info          = $e_info->textContent;
 		my $key           = $e_line->getAttribute('key');
 		my $countdown     = $e->getAttribute('countdown');
-		my $delay         = $e_info->getAttribute('delay') // 0;
+		my $delay         = $e_info->getAttribute('delay');
 		my $type          = $e_info->getAttribute('name');
 
 		my $platform_is_db = 0;
 
+		my @prev_route;
+		my @next_route;
+
+		if ( $self->{want_full_routes} ) {
+			@prev_route
+			  = $self->parse_route( @{ [ $e->findnodes($xp_prev_route) ] } );
+			@next_route
+			  = $self->parse_route( @{ [ $e->findnodes($xp_next_route) ] } );
+		}
+
 		my @line_obj
 		  = grep { $_->{identifier} eq $e_line->getAttribute('stateless') }
 		  @{ $self->{lines} };
@@ -362,7 +442,7 @@ sub results {
 			$platform_is_db = 1;
 		}
 
-		if ( $platform_name and $platform_name =~ m{ ^ (Gleis | Bstg\.)}ox ) {
+		if ( $platform_name and $platform_name =~ m{ ^ (Gleis | Bstg[.])}ox ) {
 			$platform = ( split( / /, $platform_name ) )[1];
 		}
 		elsif ( $platform_name and not $platform ) {
@@ -387,6 +467,8 @@ sub results {
 				sched_date    => $date,
 				sched_time    => $time,
 				type          => $type,
+				prev_route    => \@prev_route,
+				next_route    => \@next_route,
 			)
 		);
 	}
@@ -426,7 +508,7 @@ Travel::Status::DE::EFA - unofficial EFA departure monitor
 
 =head1 VERSION
 
-version 1.09
+version 1.10
 
 =head1 DESCRIPTION
 
@@ -478,6 +560,12 @@ B<stop> (stop/station name).
 
 address / poi / stop name to list departures for.
 
+=item B<full_routes> => B<0>|B<1>
+
+If true: Request full routes for all departures from the backend. This
+enables the B<route_pre>, B<route_post> and B<route_interesting> accessors in
+Travel::Status::DE::EFA::Result(3pm).
+
 =item B<timeout> => I<seconds>
 
 Request timeout, the argument is passed on to LWP::UserAgent(3pm).
@@ -490,6 +578,12 @@ Default: 10 seconds. Set to 0 or a negative value to disable it.
 In case of an HTTP request or EFA error, returns a string describing it. If
 none occured, returns undef.
 
+=item $status->identified_data
+
+Returns a list of the identified values for I<place> and I<name>.
+For instance, when requesting data for "E", "MartinSTR", B<identified_data>
+will return ("Essen", "Martinstr.").
+
 =item $status->lines
 
 Returns a list of Travel::Status::DE::EFA::Line(3pm) objects, each one
@@ -528,7 +622,7 @@ efa-m(1), Travel::Status::DE::EFA::Result(3pm).
 
 =head1 AUTHOR
 
-Copyright (C) 2011-2014 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
+Copyright (C) 2011-2015 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
 
 =head1 LICENSE
 
