@@ -4,9 +4,11 @@ use strict;
 use warnings;
 use 5.010;
 
+no if $] >= 5.018, warnings => 'experimental::smartmatch';
+
 use parent 'Class::Accessor';
 
-our $VERSION = '1.09';
+our $VERSION = '1.10';
 
 Travel::Status::DE::EFA::Result->mk_ro_accessors(
 	qw(countdown date delay destination is_cancelled info key line lineref
@@ -18,7 +20,7 @@ sub new {
 
 	my $ref = \%conf;
 
-	if ( $ref->{delay} eq '-9999' ) {
+	if ( defined $ref->{delay} and $ref->{delay} eq '-9999' ) {
 		$ref->{delay}        = 0;
 		$ref->{is_cancelled} = 1;
 	}
@@ -29,6 +31,71 @@ sub new {
 	return bless( $ref, $obj );
 }
 
+sub route_pre {
+	my ($self) = @_;
+
+	return @{ $self->{prev_route} };
+}
+
+sub route_post {
+	my ($self) = @_;
+
+	return @{ $self->{next_route} };
+}
+
+sub route_interesting {
+	my ( $self, $max_parts ) = @_;
+
+	my @via = $self->route_post;
+	my ( @via_main, @via_show, $last_stop );
+	$max_parts //= 3;
+
+	for my $stop (@via) {
+		if (
+			$stop->name_suf =~ m{ Bf | Hbf | Flughafen | Hauptbahnhof
+				| Krankenhaus | Klinik | (?: S $ ) }ox
+		  )
+		{
+			push( @via_main, $stop );
+		}
+	}
+	$last_stop = pop(@via);
+
+	if ( @via_main and $via_main[-1] == $last_stop ) {
+		pop(@via_main);
+	}
+	if ( @via and $via[-1] == $last_stop ) {
+		pop(@via);
+	}
+
+	if ( @via_main and @via and $via[0] == $via_main[0] ) {
+		shift(@via_main);
+	}
+
+	if ( @via < $max_parts ) {
+		@via_show = @via;
+	}
+	else {
+		if ( @via_main >= $max_parts ) {
+			@via_show = ( $via[0] );
+		}
+		else {
+			@via_show = splice( @via, 0, $max_parts - @via_main );
+		}
+
+		while ( @via_show < $max_parts and @via_main ) {
+			my $stop = shift(@via_main);
+			# FIXME cannot smartmatch $stop since it became an object
+#			if ( $stop ~~ \@via_show or $stop == $last_stop ) {
+#				next;
+#			}
+			push( @via_show, $stop );
+		}
+	}
+
+	return @via_show;
+}
+
 sub TO_JSON {
 	my ($self) = @_;
 
@@ -56,7 +123,7 @@ departure received by Travel::Status::DE::EFA
 
 =head1 VERSION
 
-version 1.09
+version 1.10
 
 =head1 DESCRIPTION
 
@@ -86,7 +153,7 @@ Actual departure date (DD.MM.YYYY).
 =item $departure->delay
 
 Expected delay from scheduled departure time in minutes. A delay of 0
-indicates either departure on time or that no delay information is available.
+indicates departure on time. undef when no realtime information is available.
 
 =item $departure->destination
 
@@ -131,6 +198,22 @@ otherwise.
 Unfortunately, there is no distinction between tram and bus platforms yet,
 which also may have the same numbers.
 
+=item $departure->route_interesting
+
+List of up to three "interesting" stations served by this departure. Is a
+subset of B<route_post>. Each station is a Travel::Status::DE::EFA::Stop(3pm)
+object.
+
+=item $departure->route_pre
+
+List of stations the train passed (or will have passed) befoe this stop.
+Each station is a Travel::Status::DE::EFA::Stop(3pm) object.
+
+=item $departure->route_post
+
+List of stations the train will pass after this stop.
+Each station is a Travel::Status::DE::EFA::Stop(3pm) object.
+
 =item $departure->sched_date
 
 Scheduled departure date (DD.MM.YYYY).
@@ -218,7 +301,7 @@ Travel::Status::DE::EFA(3pm).
 
 =head1 AUTHOR
 
-Copyright (C) 2011-2014 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
+Copyright (C) 2011-2015 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
 
 =head1 LICENSE
 
